# -*- coding: utf-8 -*-

"""Top-level package for scheduler_tools."""

__author__ = "Gregory Johnson"
__email__ = "gregj@alleninstitute.org"
# Do not edit this string manually, always use bumpversion
# Details in CONTRIBUTING.md
__version__ = "0.1.5"


def get_module_version():
    return __version__


from .example import Example  # noqa: F401
from .connector import Connector  # noqa: F401

